@@ -127,6 +127,7 @@ enum {
 	R_CONS_PAL_GUI_BACKGROUND,
 	R_CONS_PAL_GUI_ALT_BACKGROUND,
 	R_CONS_PAL_GUI_BORDER,
+	R_CONS_PAL_LINE_HIGHLIGHT,
 	R_CONS_PAL_GRAPH_BOX,
 	R_CONS_PAL_GRAPH_BOX2,
 	R_CONS_PAL_GRAPH_BOX3,
@@ -207,6 +208,7 @@ typedef struct r_cons_palette_t {
 	RColor gui_alt_background;
 	RColor gui_border;
 	RColor highlight;
+	RColor line_highlight;
 
 	/* Graph colors */
 	RColor graph_box;
@@ -271,6 +273,7 @@ typedef struct r_cons_printable_palette_t {
 	char *gui_alt_background;
 	char *gui_border;
 	char *highlight;
+	char *line_highlight;
 
 	/* graph colors */
 	char *graph_box;
@@ -366,6 +369,7 @@ typedef struct r_cons_canvas_t {
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
+typedef void (*RConsBreakCallback)(void *core);
 
 typedef struct r_cons_t {
 	RConsGrep grep;
@@ -400,7 +404,10 @@ typedef struct r_cons_t {
 	void *event_data;
 	int mouse_event;
 
-	RConsEditorCallback editor;
+	RConsEditorCallback cb_editor;
+	RConsBreakCallback cb_break;
+	RConsClickCallback cb_click;
+
 	void *user; // Used by <RCore*>
 #if __UNIX__ || __CYGWIN__ && !defined(MINGW32)
 	struct termios term_raw, term_buf;
@@ -422,7 +429,6 @@ typedef struct r_cons_t {
 	struct r_line_t *line;
 	const char **vline;
 	int refcnt;
-	RConsClickCallback onclick;
 	bool newline;
 #if __WINDOWS__ && !__CYGWIN__
 	bool ansicon;
@@ -438,7 +444,7 @@ typedef struct r_cons_t {
 	bool grep_color;
 	bool use_tts;
 	bool filter;
-	char* (*rgbstr)(char *str, ut64 addr);
+	char* (*rgbstr)(char *str, size_t sz, ut64 addr);
 } RCons;
 
 // XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
@@ -471,9 +477,9 @@ typedef struct r_cons_t {
 #define Color_INVERT       "\x1b[7m"
 #define Color_INVERT_RESET "\x1b[27m"
 /* plain colors */
-#define Color_RESET      "\x1b[0m"
-#define Color_NOBGRESET  "\x1b[22;24;25;27;28;39m"
-#define Color_BGRESET    "\x1b[49m"
+#define Color_RESET      "\x1b[0m" /* reset all */
+#define Color_RESET_NOBG "\x1b[22;24;25;27;28;39m"  /* Reset everything except background */
+#define Color_RESET_BG   "\x1b[49m"
 #define Color_BLACK      "\x1b[30m"
 #define Color_BGBLACK    "\x1b[40m"
 #define Color_RED        "\x1b[31m"
@@ -735,8 +741,8 @@ R_API int r_cons_grepbuf(char *buf, int len);
 R_API void r_cons_rgb(ut8 r, ut8 g, ut8 b, ut8 a);
 R_API void r_cons_rgb_fgbg(ut8 r, ut8 g, ut8 b, ut8 R, ut8 G, ut8 B);
 R_API void r_cons_rgb_init(void);
-R_API char *r_cons_rgb_str(char *outstr, RColor *rcolor);
-R_API char *r_cons_rgb_str_off(char *outstr, ut64 off);
+R_API char *r_cons_rgb_str(char *outstr, size_t sz, RColor *rcolor);
+R_API char *r_cons_rgb_str_off(char *outstr, size_t sz, ut64 off);
 R_API void r_cons_color(int fg, int r, int g, int b);
 R_API RColor r_cons_color_random(ut8 alpha);
 R_API void r_cons_invert(int set, int color);
@@ -781,17 +787,18 @@ typedef struct r_line_comp_t {
 } RLineCompletion;
 
 typedef char* (*RLineEditorCb)(void *core, const char *str);
-
 typedef int (*RLineHistoryUpCb)(RLine* line);
 typedef int (*RLineHistoryDownCb)(RLine* line);
 
 struct r_line_t {
 	RLineCompletion completion;
-	RLineHistory history;
-	RLineHistoryUpCb history_up_cb;
-	RLineHistoryDownCb history_down_cb;
 	RLineBuffer buffer;
-	RLineEditorCb editor_cb;
+	RLineHistory history;
+	/* callbacks */
+	RLineHistoryUpCb cb_history_up;
+	RLineHistoryDownCb cb_history_down;
+	RLineEditorCb cb_editor;
+	/* state , TODO: use more bool */
 	int echo;
 	int has_echo;
 	char *prompt;
@@ -921,6 +928,7 @@ R_API void r_agraph_add_edge(const RAGraph *g, RANode *a, RANode *b);
 R_API void r_agraph_add_edge_at(const RAGraph *g, RANode *a, RANode *b, int nth);
 R_API void r_agraph_del_edge(const RAGraph *g, RANode *a, RANode *b);
 R_API void r_agraph_print(RAGraph *g);
+R_API void r_agraph_print_json(RAGraph *g);
 R_API Sdb *r_agraph_get_sdb(RAGraph *g);
 R_API void r_agraph_foreach(RAGraph *g, RANodeCallback cb, void *user);
 R_API void r_agraph_foreach_edge(RAGraph *g, RAEdgeCallback cb, void *user);
@@ -938,6 +946,7 @@ typedef struct r_panels_t {
 	int menu_y;
 	int callgraph;
 	int curnode;
+	int originCursor;
 	bool isResizing;
 } RPanels;
 #ifdef R_API
