@@ -9,7 +9,7 @@
 #undef HAVE_PTHREAD
 #define HAVE_PTHREAD 0
 #define R_TH_TID HANDLE
-#define R_TH_LOCK_T PCRITICAL_SECTION
+#define R_TH_LOCK_T CRITICAL_SECTION
 //HANDLE
 
 #elif HAVE_PTHREAD
@@ -35,6 +35,10 @@ typedef struct r_th_lock_t {
 
 typedef struct r_th_t {
 	R_TH_TID tid;
+#if HAVE_PTHREAD
+	pthread_mutex_t _mutex;
+	pthread_cond_t _cond;
+#endif
 	RThreadLock *lock;
 	R_TH_FUNCTION(fun);
 	void *user;    // user pointer
@@ -51,14 +55,17 @@ typedef struct r_th_pool_t {
 
 #ifdef R_API
 R_API RThread *r_th_new(R_TH_FUNCTION(fun), void *user, int delay);
-R_API int r_th_start(RThread *th, int enable);
+R_API bool r_th_start(RThread *th, int enable);
 R_API int r_th_wait(RThread *th);
 R_API int r_th_wait_async(RThread *th);
 R_API void r_th_break(RThread *th);
 R_API void *r_th_free(RThread *th);
-R_API int r_th_kill(struct r_th_t *th, int force);
+R_API bool r_th_kill(RThread *th, bool force);
+R_API bool r_th_pause(RThread *th, bool enable);
+R_API bool r_th_try_pause(RThread *th);
+R_API R_TH_TID r_th_self(void);
 
-R_API RThreadLock *r_th_lock_new(void);
+R_API RThreadLock *r_th_lock_new(bool recursive);
 R_API int r_th_lock_wait(RThreadLock *th);
 R_API int r_th_lock_check(RThreadLock *thl);
 R_API int r_th_lock_enter(RThreadLock *thl);
